from enum import StrEnum


class HttpHeaders(StrEnum):
    AIM = "A-IM"
    Accept = "Accept"
    AcceptAdditions = "Accept-Additions"
    AcceptCH = "Accept-CH"
    AcceptCharset = "Accept-Charset"
    AcceptDatetime = "Accept-Datetime"
    AcceptEncoding = "Accept-Encoding"
    AcceptFeatures = "Accept-Features"
    AcceptLanguage = "Accept-Language"
    AcceptPatch = "Accept-Patch"
    AcceptPost = "Accept-Post"
    AcceptRanges = "Accept-Ranges"
    AcceptSignature = "Accept-Signature"
    AccessControl = "Access-Control"
    AccessControlAllowCredentials = "Access-Control-Allow-Credentials"
    AccessControlAllowHeaders = "Access-Control-Allow-Headers"
    AccessControlAllowMethods = "Access-Control-Allow-Methods"
    AccessControlAllowOrigin = "Access-Control-Allow-Origin"
    AccessControlExposeHeaders = "Access-Control-Expose-Headers"
    AccessControlMaxAge = "Access-Control-Max-Age"
    AccessControlRequestHeaders = "Access-Control-Request-Headers"
    AccessControlRequestMethod = "Access-Control-Request-Method"
    Age = "Age"
    Allow = "Allow"
    ALPN = "ALPN"
    AltSvc = "Alt-Svc"
    AltUsed = "Alt-Used"
    Alternates = "Alternates"
    AMPCacheTransform = "AMP-Cache-Transform"
    ApplyToRedirectRef = "Apply-To-Redirect-Ref"
    AuthenticationControl = "Authentication-Control"
    AuthenticationInfo = "Authentication-Info"
    Authorization = "Authorization"
    CExt = "C-Ext"
    CMan = "C-Man"
    COpt = "C-Opt"
    CPEP = "C-PEP"
    CPEPInfo = "C-PEP-Info"
    CacheControl = "Cache-Control"
    CacheStatus = "Cache-Status"
    CalManagedID = "Cal-Managed-ID"
    CalDAVTimezones = "CalDAV-Timezones"
    CapsuleProtocol = "Capsule-Protocol"
    CDNCacheControl = "CDN-Cache-Control"
    CDNLoop = "CDN-Loop"
    CertNotAfter = "Cert-Not-After"
    CertNotBefore = "Cert-Not-Before"
    ClearSiteData = "Clear-Site-Data"
    ClientCert = "Client-Cert"
    ClientCertChain = "Client-Cert-Chain"
    Close = "Close"
    ConfigurationContext = "Configuration-Context"
    Connection = "Connection"
    ContentBase = "Content-Base"
    ContentDigest = "Content-Digest"
    ContentDisposition = "Content-Disposition"
    ContentEncoding = "Content-Encoding"
    ContentID = "Content-ID"
    ContentLanguage = "Content-Language"
    ContentLength = "Content-Length"
    ContentLocation = "Content-Location"
    ContentMD5 = "Content-MD5"
    ContentRange = "Content-Range"
    ContentScriptType = "Content-Script-Type"
    ContentSecurityPolicy = "Content-Security-Policy"
    ContentSecurityPolicyReportOnly = "Content-Security-Policy-Report-Only"
    ContentStyleType = "Content-Style-Type"
    ContentType = "Content-Type"
    ContentVersion = "Content-Version"
    Cookie = "Cookie"
    Cookie2 = "Cookie2"
    CrossOriginEmbedderPolicy = "Cross-Origin-Embedder-Policy"
    CrossOriginEmbedderPolicyReportOnly = "Cross-Origin-Embedder-Policy-Report-Only"
    CrossOriginOpenerPolicy = "Cross-Origin-Opener-Policy"
    CrossOriginOpenerPolicyReportOnly = "Cross-Origin-Opener-Policy-Report-Only"
    CrossOriginResourcePolicy = "Cross-Origin-Resource-Policy"
    DASL = "DASL"
    Date = "Date"
    DAV = "DAV"
    DefaultStyle = "Default-Style"
    DeltaBase = "Delta-Base"
    Depth = "Depth"
    DerivedFrom = "Derived-From"
    Destination = "Destination"
    DifferentialID = "Differential-ID"
    Digest = "Digest"
    DPoP = "DPoP"
    DPoPNonce = "DPoP-Nonce"
    EarlyData = "Early-Data"
    EDIINTFeatures = "EDIINT-Features"
    ETag = "ETag"
    Expect = "Expect"
    ExpectCT = "Expect-CT"
    Expires = "Expires"
    Ext = "Ext"
    Forwarded = "Forwarded"
    From = "From"
    GetProfile = "GetProfile"
    Hobareg = "Hobareg"
    Host = "Host"
    HTTP2Settings = "HTTP2-Settings"
    If = "If"
    IfMatch = "If-Match"
    IfModifiedSince = "If-Modified-Since"
    IfNoneMatch = "If-None-Match"
    IfRange = "If-Range"
    IfScheduleTagMatch = "If-Schedule-Tag-Match"
    IfUnmodifiedSince = "If-Unmodified-Since"
    IM = "IM"
    IncludeReferredTokenBindingID = "Include-Referred-Token-Binding-ID"
    Isolation = "Isolation"
    KeepAlive = "Keep-Alive"
    Label = "Label"
    LastEventID = "Last-Event-ID"
    LastModified = "Last-Modified"
    Link = "Link"
    Location = "Location"
    LockToken = "Lock-Token"
    Man = "Man"
    MaxForwards = "Max-Forwards"
    MementoDatetime = "Memento-Datetime"
    Meter = "Meter"
    MethodCheck = "Method-Check"
    MethodCheckExpires = "Method-Check-Expires"
    MIMEVersion = "MIME-Version"
    Negotiate = "Negotiate"
    NEL = "NEL"
    ODataEntityId = "OData-EntityId"
    ODataIsolation = "OData-Isolation"
    ODataMaxVersion = "OData-MaxVersion"
    ODataVersion = "OData-Version"
    Opt = "Opt"
    OptionalWWWAuthenticate = "Optional-WWW-Authenticate"
    OrderingType = "Ordering-Type"
    Origin = "Origin"
    OriginAgentCluster = "Origin-Agent-Cluster"
    OSCORE = "OSCORE"
    OSLCCoreVersion = "OSLC-Core-Version"
    Overwrite = "Overwrite"
    P3P = "P3P"
    PEP = "PEP"
    PEPInfo = "PEP-Info"
    PermissionsPolicy = "Permissions-Policy"
    PICSLabel = "PICS-Label"
    PingFrom = "Ping-From"
    PingTo = "Ping-To"
    Position = "Position"
    Pragma = "Pragma"
    Prefer = "Prefer"
    PreferenceApplied = "Preference-Applied"
    Priority = "Priority"
    ProfileObject = "ProfileObject"
    Protocol = "Protocol"
    ProtocolInfo = "Protocol-Info"
    ProtocolQuery = "Protocol-Query"
    ProtocolRequest = "Protocol-Request"
    ProxyAuthenticate = "Proxy-Authenticate"
    ProxyAuthenticationInfo = "Proxy-Authentication-Info"
    ProxyAuthorization = "Proxy-Authorization"
    ProxyFeatures = "Proxy-Features"
    ProxyInstruction = "Proxy-Instruction"
    ProxyStatus = "Proxy-Status"
    Public = "Public"
    PublicKeyPins = "Public-Key-Pins"
    PublicKeyPinsReportOnly = "Public-Key-Pins-Report-Only"
    Range = "Range"
    RedirectRef = "Redirect-Ref"
    Referer = "Referer"
    RefererRoot = "Referer-Root"
    Refresh = "Refresh"
    RepeatabilityClientID = "Repeatability-Client-ID"
    RepeatabilityFirstSent = "Repeatability-First-Sent"
    RepeatabilityRequestID = "Repeatability-Request-ID"
    RepeatabilityResult = "Repeatability-Result"
    ReplayNonce = "Replay-Nonce"
    ReportingEndpoints = "Reporting-Endpoints"
    ReprDigest = "Repr-Digest"
    RetryAfter = "Retry-After"
    Safe = "Safe"
    ScheduleReply = "Schedule-Reply"
    ScheduleTag = "Schedule-Tag"
    SecGPC = "Sec-GPC"
    SecPurpose = "Sec-Purpose"
    SecTokenBinding = "Sec-Token-Binding"
    SecWebSocketAccept = "Sec-WebSocket-Accept"
    SecWebSocketExtensions = "Sec-WebSocket-Extensions"
    SecWebSocketKey = "Sec-WebSocket-Key"
    SecWebSocketProtocol = "Sec-WebSocket-Protocol"
    SecWebSocketVersion = "Sec-WebSocket-Version"
    SecurityScheme = "Security-Scheme"
    Server = "Server"
    ServerTiming = "Server-Timing"
    SetCookie = "Set-Cookie"
    SetCookie2 = "Set-Cookie2"
    SetProfile = "SetProfile"
    Signature = "Signature"
    SignatureInput = "Signature-Input"
    SLUG = "SLUG"
    SoapAction = "SoapAction"
    StatusURI = "Status-URI"
    StrictTransportSecurity = "Strict-Transport-Security"
    Sunset = "Sunset"
    SurrogateCapability = "Surrogate-Capability"
    SurrogateControl = "Surrogate-Control"
    TCN = "TCN"
    TE = "TE"
    Timeout = "Timeout"
    TimingAllowOrigin = "Timing-Allow-Origin"
    Topic = "Topic"
    Traceparent = "Traceparent"
    Tracestate = "Tracestate"
    Trailer = "Trailer"
    TransferEncoding = "Transfer-Encoding"
    TTL = "TTL"
    Upgrade = "Upgrade"
    Urgency = "Urgency"
    URI = "URI"
    UserAgent = "User-Agent"
    VariantVary = "Variant-Vary"
    Vary = "Vary"
    Via = "Via"
    WantContentDigest = "Want-Content-Digest"
    WantDigest = "Want-Digest"
    WantReprDigest = "Want-Repr-Digest"
    Warning = "Warning"
    WWWAuthenticate = "WWW-Authenticate"
    XContentTypeOptions = "X-Content-Type-Options"
    XFrameOptions = "X-Frame-Options"
